/***
 * Juliac
 * Copyright (C) 2017-2021 Inria, Univ. Lille
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.api.generator;

import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.api.desc.MembraneDesc;

/**
 * Interface for setting {@link InterfaceType}-related information in source
 * code generators.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille.fr>
 * @since 2.7
 */
public interface InterfaceTypeConfigurableItf extends ClassGeneratorItf {

	/**
	 * Return <code>true</code> if some code must be generated by this
	 * generator.
	 */
	public boolean match();

	/**
	 * Setter method for the {@link InterfaceType} associated to this generator.
	 *
	 * @param it       the interface type
	 * @param proxycl  the class associated to the interface type
	 * @param pkgRoot
	 *         the root package name for the generated proxy class with a trailing
	 *         dot or the empty string
	 */
	public void setInterfaceType( InterfaceType it, Object proxycl, String pkgRoot );

	/**
	 * Setter method for the membrane descriptor associated to this generator.
	 */
	public void setMembraneDesc( MembraneDesc<?> membraneDesc );

	/**
	 * Setter method for specifying whether the generated code can be merged.
	 */
	public void setMergeable( boolean mergeable );
}
