/***
 * Juliac
 * Copyright (C) 2015-2021 Inria, Univ. Lille
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.api.generator;

import java.io.IOException;

import org.objectweb.fractal.juliac.api.visit.FileSourceCodeVisitor;

/**
 * Interface for source code generators.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille.fr>
 * @since 2.6
 */
public interface SourceCodeGeneratorItf {

	/**
	 * Generate the code in the specified {@link FileSourceCodeVisitor}.
	 *
	 * @param fv  the visitor for generating the source code
	 */
	public void generate( FileSourceCodeVisitor fv ) throws IOException;

	/**
	 * Return the name of the type generated by this generator.
	 */
	public String getTargetTypeName();

	/**
	 * Return the code element that is the source for the type generated by this
	 * generator.
	 *
	 * @since 2.7
	 */
	public Object getSourceType();
}
