/***
 * Juliac
 * Copyright (C) 2015-2021 Inria, Univ. Lille
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.core.opt;

import java.lang.reflect.Modifier;
import java.util.Date;

import org.objectweb.fractal.juliac.api.generator.TypeGeneratorItf;
import org.objectweb.fractal.juliac.api.visit.FileSourceCodeVisitor;

/**
 * Default implementation for type generators.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille.fr>
 * @since 2.6
 */
public abstract class TypeGenerator implements TypeGeneratorItf {

	@Override
	public void generate( FileSourceCodeVisitor fv ) {
		generateFileHeaders(fv);
		String packageName = getPackageName();
		if( packageName != null ) {
			fv.visitPackageName(packageName);
		}
		generateImports(fv);
	}

	@Override
	public void generateFileHeaders( FileSourceCodeVisitor fv ) {
		fv.visitComment("/*");
		fv.visitComment(" * Generated by: "+getClass().getName());
		fv.visitComment(" * on: "+new Date().toString());
		fv.visitComment(" */");
	}

	@Override
	public String getPackageName() {
		String targettypename = getTargetTypeName();
		int lastdot = targettypename.lastIndexOf('.');
		if( lastdot == -1 ) {
			return null;
		}
		String packagename = targettypename.substring(0,lastdot);
		return packagename;
	}

	@Override
	public void generateImports( FileSourceCodeVisitor fv ) {}

	@Override
	public int getTypeModifiers() {
		return Modifier.PUBLIC;
	}

	@Override
   public String getSimpleTypeName() {
		String targettypename = getTargetTypeName();
		int lastdot = targettypename.lastIndexOf('.');
		String shorttypename = targettypename.substring(lastdot+1);
		return shorttypename;
	}

	@Override
	public String[] getGenericTypeParameters() { return null; }

	@Override
	public Object getSourceType() {
		return null;
	}
}
