/***
 * Juliac
 * Copyright (C) 2007-2021 Inria, Univ. Lille
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.core.proxy;

import org.objectweb.fractal.api.type.InterfaceType;

/**
 * Defines the legal modes for an interceptor generated by {@link
 * SimpleSourceCodeGenerator}. The mode defines whether the interceptor applies
 * on input interfaces, output interfaces or both.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille.fr>
 */
public enum SimpleSourceCodeGeneratorMode {

	IN {
		public boolean match( InterfaceType it ) {
			return ! it.isFcClientItf();
		}
	},

	IN_OUT {
		public boolean match( InterfaceType it ) {
			return true;
		}
	},

	OUT {
		public boolean match( InterfaceType it ) {
			return it.isFcClientItf();
		}
	};

	public abstract boolean match( InterfaceType it );
}
