/***
 * Juliac
 * Copyright (C) 2019-2021 Inria, Univ. Lille
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.adlet.opt.comp;

import javax.lang.model.element.TypeElement;

import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.api.desc.MembraneDesc;

/**
 * This class generates the source code of lifecycle interceptors for control
 * components. This class extends the regular lifecycle generator to enable the
 * generation of interceptors for business interfaces of control components.
 * These business interfaces correspond to the control interfaces of business
 * components and have a name which starts with //
 *
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille.fr>
 * @since 2.8
 */
public class LifeCycleSourceCodeGenerator
extends org.objectweb.fractal.juliac.adlet.core.proxy.LifeCycleSourceCodeGenerator {

	public LifeCycleSourceCodeGenerator() {
		super();
	}

	public LifeCycleSourceCodeGenerator(
		InterfaceType it, TypeElement cl, String pkgRoot,
		MembraneDesc<?> membraneDesc, boolean mergeable ) {

		super(it,cl,pkgRoot,membraneDesc,mergeable);
	}

	/**
	 * Return <code>true</code> if some code must be generated by the current
	 * source code generator.
	 */
	@Override
	public boolean match() {
		if( it.isFcClientItf() ) {
			return false;   // Skip client interfaces
		}
		String name = it.getFcItfName();
		if( name.startsWith("//") ) {
			return true;    // Match Koch meta interfaces
		}
		if( name.endsWith("-controller") || name.equals("component") ) {
			return false;   // Skip control interfaces
		}
		return true;
	}
}
