/***
 * Juliac
 * Copyright (C) 2007-2021 Inria, Univ. Lille
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.opt.mergeall;

import org.objectweb.fractal.juliac.api.generator.InterceptorSourceCodeGeneratorItf;
import org.objectweb.fractal.juliac.core.proxy.InterceptorClassGenerator;

/**
 * This class is a {@link InterceptorClassGenerator} which aggregates
 * the source code generated by several {@link
 * InterceptorSourceCodeGeneratorItf}s.
 *
 * The generated class has an empty package name which makes it suitable for
 * embedding as an inner class.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille.fr>
 */
public class InnerInterceptorClassGenerator
extends InterceptorClassGenerator {

	public InnerInterceptorClassGenerator() {
		super();
	}

	/**
	 * Return the name of the class generated by this generator.
	 * Return a name with an empty package name.
	 */
	@Override
	public String getTargetTypeName() {
		String javaItfSignature = it.getFcItfSignature();
		int lastdot = javaItfSignature.lastIndexOf('.');
		String name =
			(lastdot==-1) ?
			javaItfSignature :
			javaItfSignature.substring(lastdot+1);
		String suffix = getClassNameSuffix();
		String targetClassname = name + suffix;
		return targetClassname;
	}
}
