/***
 * Juliac
 * Copyright (C) 2009-2021 Inria, Univ. Lille
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 * Contributor: Romain Rouvoy
 */

package org.objectweb.fractal.juliac.osgi;

import java.lang.reflect.Modifier;

import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.api.visit.BlockSourceCodeVisitor;
import org.objectweb.fractal.juliac.api.visit.ClassSourceCodeVisitor;
import org.objectweb.fractal.juliac.core.opt.ClassGenerator;
import org.objectweb.fractal.juliac.osgi.revision.api.OSGiRevisionItf;

/**
 * This class generates OSGi activator classes for stubs generated by {@link
 * DynServiceClassGenerator}.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille.fr>
 * @author Romain Rouvoy <Romain.Rouvoy@univ-lille.fr>
 * @since 2.1.6
 */
public class DynServiceActivatorClassGenerator extends ClassGenerator {

	private String pkgName;
	private String targetClassName;
	private InterfaceType it;

	public DynServiceActivatorClassGenerator(
		String pkgName, String targetClassName, InterfaceType it ) {

		this.pkgName = pkgName;
		this.targetClassName = targetClassName;
		this.it = it;
	}

	public String getTargetTypeName() {
		return pkgName+".Activator";
	}

	@Override
	public String[] getImplementedInterfaceNames() {
		return new String[]{OSGiRevisionItf.BUNDLEACTIVATOR_CLASSNAME};
	}

	@Override
	public void generateMethods( ClassSourceCodeVisitor cv ) {

		/*
		 * Method start(BundleContext)
		 */
		BlockSourceCodeVisitor mv =
			cv.visitMethod(
				Modifier.PUBLIC, null, "void", "start",
				new String[]{OSGiRevisionItf.BUNDLECONTEXT_CLASSNAME+" context"},
				null );
		mv.visit  ("    context.registerService( \"");
		mv.visit  (it.getFcItfSignature());
		mv.visit  ("\", new ");
		mv.visit  (targetClassName);
		mv.visitln("(), null);");
		mv.visitEnd();

		/*
		 * Method stop(BundleContext)
		 */
		mv =
			cv.visitMethod(
				Modifier.PUBLIC, null, "void", "stop",
				new String[]{OSGiRevisionItf.BUNDLECONTEXT_CLASSNAME+" context"},
				null );
		mv.visitEnd();
	}
}
