/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2011-2021 Inria, Univ. Lille
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.koch.control.interceptor

import org.objectweb.fractal.julia.ComponentInterface
import org.objectweb.fractal.julia.Interceptor
import org.objectweb.fractal.julia.BasicControllerTrait
import org.objectweb.fractal.julia.InitializationContext
import org.objectweb.fractal.api.`type`.InterfaceType
import org.objectweb.fractal.api.control.BindingController
import org.objectweb.fractal.api.control.LifeCycleController
import org.objectweb.fractal.api.control.IllegalLifeCycleException
import org.objectweb.fractal.julia.control.lifecycle.UseLifeCycleControllerTrait
import org.objectweb.fractal.julia.control.name.UseNameControllerTrait

/**
 * Trait implementing the functionalities of the {@link InterceptorController}
 * control interface.
 *
 * @author Lionel <Lionel.Seinturier@univ-lille.fr>
 * @since 1.4.5
 */
trait LifeCycleInterceptorTrait extends AbstractInterceptorControllerTrait
with UseLifeCycleControllerTrait with UseNameControllerTrait {

	override def addFcInterceptor( itf: ComponentInterface, interceptor: Interceptor ) = {
		checkFcStopped
		super.addFcInterceptor(itf,interceptor)
	}

	override def removeFcInterceptor( itf: ComponentInterface, interceptor: Interceptor ) : Boolean = {
		checkFcStopped
		return super.removeFcInterceptor(itf,interceptor)
	}

	// -----------------------------------------------------------------------
	// Implementation specific
	// -----------------------------------------------------------------------

	private def checkFcStopped = {
		val state = weaveableLC.getFcState
		if( ! LifeCycleController.STOPPED.equals(state) ) {
			val msg = "Component "+weaveableNC.getFcName+" is not stopped"
			throw new IllegalLifeCycleException(msg)
		}
	}
}
