/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2009-2021 Inria, Univ. Lille
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.opt.oo;

import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.api.desc.MembraneDesc;
import org.objectweb.fractal.juliac.api.visit.BlockSourceCodeVisitor;
import org.objectweb.fractal.juliac.commons.lang.ClassHelper;
import org.objectweb.fractal.juliac.core.proxy.AbstractInterceptorSourceCodeGenerator;
import org.ow2.frascati.tinfi.TinfiComponentInterceptor;

/**
 * This class generates the source code that is common to all Tinfi
 * interceptors, whether or not an intent layer is used.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille.fr>
 * @since 1.3.1
 */
public class SCATinfiInterceptorSourceCodeGenerator
extends AbstractInterceptorSourceCodeGenerator {

	public SCATinfiInterceptorSourceCodeGenerator() {
		super();
	}

	public SCATinfiInterceptorSourceCodeGenerator(
		InterfaceType it, Class<?> cl, String pkgRoot,
		MembraneDesc<?> membraneDesc, boolean mergeable ) {

		super(it,cl,pkgRoot,membraneDesc,mergeable);
	}

	/**
	 * Return the name of the suffix which is appended to generated class names.
	 */
	public String getClassNameSuffix() {
		return "SCA";
	}

	/**
	 * Return <code>true</code> if some code must be generated by the current
	 * source code generator.
	 */
	public boolean match() {
		// Match all interfaces, including control ones
		return true;
	}

	@Override
	public void setMergeable( boolean mergeable ) {
		if(mergeable) {
			final String msg = "This generator cannot generate mergeable code";
			throw new IllegalArgumentException(msg);
		}
	}

	@Override
	public String getSuperClassName() {

		StringBuilder sb = new StringBuilder();
		sb.append(TinfiComponentInterceptor.class.getName());
		sb.append('<');

		String signature = it.getFcItfSignature();
		sb.append(signature);

		/*
		 * Add type parameters if any.
		 */
		String[] tpnames = ClassHelper.getTypeParameterNames(proxycl);
		if( tpnames.length != 0 ) {
			String s = ClassHelper.getTypeParameterNamesSignature(tpnames);
			sb.append(s);
		}

		sb.append('>');
		return sb.toString();
	}

	@Override
	public void generateMethodInitFcController( BlockSourceCodeVisitor mv ) {
		mv.visitIns("super.initFcController(ic)");
	}

	@Override
	public void generateMethodClone( BlockSourceCodeVisitor mv ) {
		mv.visitIns("initFcClone(clone)");
	}
}
